using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

enum CustomPoses
{
    one,
    two
}

public class TerminatorLimbBehaviour : MonoBehaviour 
{
    public SkinMaterialHandler SkinMaterialHandler;
    public PhysicalBehaviour PhysicalBehaviour;
    public LimbBehaviour LimbBehaviour;
    public PersonBehaviour Person;
    public GameObject Head;
    public LimbBehaviour HeadLimbBehaviour;
    private float VDamageBaseline = 320000; //5000 * 64, aka all damage point z.
    public bool ExposedEndoskeleton = false;
    public bool CPUDeath = false;
    private GameObject TEye;
    public bool InTimeBubble;
    private Rigidbody2D RigidB;
    public float StasisFade = 0;
    public SpriteRenderer SpriteRenderer;
    private float StasisFadeTimeGoal;
    private float StasisLeaveTimeGoal;
    private bool Registered = false;
    public int HeadGetExos = 0;

    private void Start(){
        ExposedEndoskeleton = false;
        HeadGetExos = 0;
        StasisFadeTimeGoal = (UnityEngine.Time.fixedTime + 2f);
        StasisLeaveTimeGoal = (UnityEngine.Time.fixedTime + 7f);
        InTimeBubble = true;
        Person = LimbBehaviour.Person;
        if ((gameObject.name == "Head")){
            Head = gameObject;
            TerminatorMusicManager.TRegist();
            Registered = true;
            TAssets.PlaySFX(TAssets.Terminator_TimeBubble, gameObject.transform.position);
            TAssets.PlaySFX(TAssets.music_init, gameObject.transform.position);
        }else{
            Head = this.gameObject.transform.root.Find("Head").gameObject;
            }
        HeadLimbBehaviour = Head.GetComponent<LimbBehaviour>();
        RigidB = gameObject.GetComponent<Rigidbody2D>();
        SpriteRenderer = gameObject.GetComponent<SpriteRenderer>();

        StasisTick();
        SetupCustomPoses();
        LimbBehaviour.Person.OverridePoseIndex = 10;
        CreateTimeBubble();
    }
    private void OnDestroy() {
        if ((Head == gameObject) && Registered){
        TerminatorMusicManager.TDeRegist();
        Registered = false;
        }
    }

    public void SetupCustomPoses(){
        RagdollPose TimeSquat = new RagdollPose();
        TimeSquat.ShouldStandUpright = false;
        LimbBehaviour[] EveryPoseLimb;
        EveryPoseLimb = gameObject.transform.root.GetComponentsInChildren<LimbBehaviour>();

        List<RagdollPose.LimbPose> Angles = new List<RagdollPose.LimbPose>();
        foreach (LimbBehaviour PoseyLimb in EveryPoseLimb){
            float Angle = 0;
            switch(PoseyLimb.gameObject.name) {//pose limb(s) angle information
            case "Head": Angle = 30; break;
            case "UpperBody": Angle = 45; break;
            case "LowerBody": Angle = -45; break;
            case "UpperLegFront": Angle = -45; break;
            case "LowerLegFront": Angle = 135; break;
            case "FootFront": Angle = -155; break;
            case "UpperLeg": Angle = -80; break;
            case "LowerLeg": Angle = 100; break;
            case "Foot": Angle = -10; break;
            case "UpperArmFront": Angle = -90; break;
            case "UpperArm": Angle = -90; break;
            default: Angle = 0; break;
            }
            RagdollPose.LimbPose NewPose = new RagdollPose.LimbPose(PoseyLimb, Angle){};
            NewPose.Angle = Angle;
            NewPose.Limb = PoseyLimb;

            Angles.Add(NewPose);
        }
        TimeSquat.Angles = Angles;
        
        LimbBehaviour.Person.Poses.Add(TimeSquat);

        foreach (RagdollPose pose in LimbBehaviour.Person.Poses){
            pose.ConstructDictionary();
        }
        //Debug.Log("CustomPoseLogged" + LimbBehaviour.Person.ActivePose.Name);
    }

    public void StasisTick(){ //tick of inside time bubbles
        if ((UnityEngine.Time.fixedTime > StasisFadeTimeGoal) && (StasisFade < 1)){
            StasisFade += 0.008f;
        }
        if (InTimeBubble){
            if (UnityEngine.Random.Range(0,(230 - (int)Mathf.Round(StasisFade * 100))) == 0)
            ModAPI.CreateParticleEffect("Spark", gameObject.transform.position);

            SpriteRenderer.color = new Color (1, 1, 1, StasisFade); 
            RigidB.velocity *= 0.3f;
            if (UnityEngine.Time.fixedTime > StasisLeaveTimeGoal){
                InTimeBubble = false;
                LimbBehaviour.Person.OverridePoseIndex = -1;
            }
        }
    }

    public void FixedUpdate()
    {
        StasisTick();

        if (ExposedEndoskeleton){ //fun fact, I use 2 If statements here cause it means it's processing less from ram in the check therefor better generally. I could use a return like ZOOI does, but that's dumb and cringe
            if (!(CapCheckVisualDamage(80) || (PhysicalBehaviour.BurnProgress > 0.9)))
            {
                UnexposeEndo();
            }
        }else{
            if (CapCheckVisualDamage(80) || (PhysicalBehaviour.BurnProgress > 0.9))
            {
                ExposeEndo();
            }
            LimbBehaviour.Health = LimbBehaviour.InitialHealth;
        }
        LimbBehaviour.Numbness = float.NegativeInfinity;
        Person.ShockLevel = 0;
        Person.PainLevel = 0;
        Person.AdrenalineLevel = 0.025f;
        Person.OxygenLevel = float.PositiveInfinity;
        LimbBehaviour.IsDismembered = false;
        PhysicalBehaviour.Charge = 0f;
        LimbBehaviour.BaseStrength = 2;
        PhysicalBehaviour.Disintegratable = false;
        LimbBehaviour.Vitality = 0f;
        LimbBehaviour.CirculationBehaviour.InternalBleedingIntensity = 0;
        PhysicalBehaviour.Wetness = 0;
        if (Head == gameObject) LimbBehaviour.HasBrain = false;
            
        if (HeadLimbBehaviour.Health < 0.1){
            Person.Consciousness = 0;
            Person.Braindead = true;
            if (!CPUDeath){ //dies
                CPUDeath = true;
                TAssets.PlaySFX(TAssets.Terminator_DeathSFX, gameObject.transform.position);
                if ((Head == gameObject) && Registered){
                    TerminatorMusicManager.TDeRegist();
                    Registered = false;
                }
            }
        }else{
            Person.Consciousness = float.PositiveInfinity;
            Person.Braindead = false;
            if (CPUDeath){ //revives somehow
                CPUDeath = false;
                if ((Head == gameObject) && !Registered){
                    TAssets.PlaySFX(TAssets.Terminator_RebootSFX, gameObject.transform.position);
                    TerminatorMusicManager.TRegist();
                    Registered = true;
                    TerminatorMusicManager.MusicProgression(2);
                }
            }
        }
        LimbBehaviour.CirculationBehaviour.IsPump = LimbBehaviour.CirculationBehaviour.WasInitiallyPumping;
        LimbBehaviour.CirculationBehaviour.BloodFlow = 100f;
        LimbBehaviour.CirculationBehaviour.ImmuneToDamage = false;
        LimbBehaviour.CirculationBehaviour.AddLiquid(LimbBehaviour.GetOriginalBloodType(), float.PositiveInfinity);
        LimbBehaviour.CirculationBehaviour.ForceSetAllLiquid(float.PositiveInfinity);

        LimbBehaviour.Frozen = false;
    }

    public void UnexposeEndo(){
        if (ExposedEndoskeleton){
            ExposedEndoskeleton = false;
            PhysicalBehaviour.Properties = ModAPI.FindPhysicalProperties("Human");
            Head.GetComponent<TerminatorLimbBehaviour>().HeadGetExos --;
        }
    }

    public void ExposeEndo(){
        if (!ExposedEndoskeleton){
            ExposedEndoskeleton = true;
            PhysicalBehaviour.Properties = ModAPI.FindPhysicalProperties("AndroidArmour");
            if (Head == gameObject){
                if (TEye == null){ //creates eye effect
                    TEye = new GameObject();
                    TEye.transform.SetParent(gameObject.transform, false);
                    SpriteRenderer TEyeRend = TEye.AddComponent<SpriteRenderer>();
                    TEyeRend.sprite = TAssets.Terminator_Eye;
                    TEyeRend.material = ModAPI.FindMaterial("VeryBright");
                    TEyeRend.sortingLayerName = "Top";
                    TerminatorEyeBehaviour TEyeComp = TEye.AddComponent<TerminatorEyeBehaviour>();
                    TEyeComp.EyeRend = TEyeRend;
                    TEyeComp.HeadLimb = HeadLimbBehaviour;
                }
            }
            Head.GetComponent<TerminatorLimbBehaviour>().HeadGetExos ++;

            if (Head.GetComponent<TerminatorLimbBehaviour>().HeadGetExos > 13){
                TerminatorMusicManager.MusicProgression(1);
            }else TerminatorMusicManager.MusicProgression(0);
        }
    }

    public float CrunchVisualDamages(){
        float CrunchOutput = 0;
        for (int i = 0; i < 64; ++i){
            CrunchOutput += SkinMaterialHandler.damagePoints[i].z;
            CrunchOutput += SkinMaterialHandler.bulletHolePoints[i].z;
        }
        return(CrunchOutput);
    }

    public bool CapCheckVisualDamage(float Cap){
        bool CapOutput = false;
        for (int i = 0; i < 64; ++i){
            if ((SkinMaterialHandler.damagePoints[i].z <= Cap) || (SkinMaterialHandler.bulletHolePoints[i].z <= Cap))
            CapOutput = true;
        }
        return(CapOutput);
    }

    public void CreateTimeBubble(){
        GameObject TimeBubble = new GameObject();
        SpriteRenderer BubbleRend = TimeBubble.AddComponent<SpriteRenderer>();
        BubbleRend.sprite = TAssets.TimeSphere;
        BubbleRend.sortingLayerName = "Top";

        TimeBubbleBehaviour BubbleBehav = TimeBubble.AddComponent<TimeBubbleBehaviour>();
        BubbleBehav.TerminatorHead = Head;
        BubbleBehav.TerminatorLimb = Head.GetComponent<TerminatorLimbBehaviour>();

        for (int f = 0;(f < 2); f++){
            GameObject TimeBubbleFlicker = new GameObject();
            SpriteRenderer BubbleRendFlick = TimeBubbleFlicker.AddComponent<SpriteRenderer>();
            BubbleRendFlick.sortingLayerName = "Top";

            TimeBubbleBehaviour BubbleBehavFlick = TimeBubbleFlicker.AddComponent<TimeBubbleBehaviour>();
            BubbleBehavFlick.TerminatorHead = Head;
            BubbleBehavFlick.TerminatorLimb = Head.GetComponent<TerminatorLimbBehaviour>();
            BubbleBehavFlick.BubbleSpriteType = 1;
        }
    }
}

public class TerminatorEyeBehaviour : MonoBehaviour 
{
    public SpriteRenderer EyeRend;
    public LimbBehaviour HeadLimb;
    public float EyeBrightness = 0.75f;
    public bool EyesDie = false;

    public void FixedUpdate(){
        EyesDie = (HeadLimb.Health < 0.1f && (gameObject.transform.parent.GetComponent<TerminatorLimbBehaviour>().ExposedEndoskeleton));

        if (EyesDie){
            if (EyeBrightness > 0){
                EyeBrightness -= 0.007f;
            }
        }else{
            if (EyeBrightness < 0.75f){
                EyeBrightness += 0.007f;
            }
        }

        EyeRend.color = new Color(EyeBrightness, EyeBrightness, EyeBrightness, 1);
    }
}

public class TimeBubbleBehaviour : MonoBehaviour 
{
    public int BubbleSpriteType = 0;
    public SpriteRenderer BubbleRend;
    public GameObject TerminatorHead;
    public TerminatorLimbBehaviour TerminatorLimb;
    private float BubbleFlickerTimer = 0;
    private float BubbleFlickerer = 0;
    public float BurnAlpha = 1;

    public void Start(){
        BubbleRend = gameObject.GetComponent<SpriteRenderer>();
        BubbleRend.color = new Color(1, 1, 1, 0);
        gameObject.transform.localScale = new Vector2(1.1f,1.1f);
        if (BubbleSpriteType == 2){
            //SpriteRenderer.maskInteraction = SpriteMaskInteraction.VisibleInsideMask;
            //UnityEngine.SpriteMask Mask = gameObject.AddComponent<UnityEngine.SpriteMask>();
            //Mask.sprite = TAssets.TimeSphere_Thermal;
        }
    }

    public void FixedUpdate(){
        if (BubbleSpriteType == 2){ //burning decals
            BubbleRend.color = new Color(1, 1, 1, (TerminatorLimb.StasisFade * BurnAlpha));
            if (TerminatorLimb.InTimeBubble)
                gameObject.transform.position = GetAllBounds(TerminatorHead.transform.root.gameObject).center;
        }else{//normal aka 0
            BubbleRend.color = new Color(1, 1, 1, (TerminatorLimb.StasisFade * 0.062f));
            gameObject.transform.position = GetAllBounds(TerminatorHead.transform.root.gameObject).center;
            if (!TerminatorLimb.InTimeBubble) UnityEngine.Object.Destroy(gameObject);
        }

        if (BubbleSpriteType == 1){
            if (UnityEngine.Time.fixedTime > BubbleFlickerTimer){
                BubbleFlickerTimer = (UnityEngine.Time.fixedTime + UnityEngine.Random.Range(0.1f,1.5f));
                BubbleRend.sprite = TAssets.TimeSphere_Flicker[UnityEngine.Random.Range(0,8)];
                BubbleFlickerer = (UnityEngine.Time.fixedTime + 0.025f);
            }
            if (UnityEngine.Time.fixedTime <= BubbleFlickerer){
                BubbleRend.color = new Color(1, 1, 1, (TerminatorLimb.StasisFade * 0.15f));
            }else{
                BubbleRend.color = new Color(1, 1, 1, 0);
            }
        }

        if (BubbleSpriteType == 0){
            Collider2D[] TimebubbleCircleHits = Physics2D.OverlapCircleAll(gameObject.transform.position, 1.05f);
            
            if ((TimebubbleCircleHits.Length > 0) && (TerminatorLimb.StasisFade > 0.35f)){

                for (int i = 0; i < TimebubbleCircleHits.Length; i++)
                {
                    if (TimebubbleCircleHits[i].gameObject.name != "Root" && (TimebubbleCircleHits[i].gameObject.GetComponent<TerminatorLimbBehaviour>() == null)) {
                    PhysicalBehaviour HitPhys = TimebubbleCircleHits[i].gameObject.GetComponent<PhysicalBehaviour>();
                    //HitPhys.Ignite(true);
                    HitPhys.Temperature = 150;
                    HitPhys.BurnProgress = 0.5f;
                    }
                }

            }
        }
    }

    private Bounds GetAllBounds(GameObject It){
        Collider2D[] Colliders = It.GetComponentsInChildren<Collider2D>();

        if (Colliders.Length > 0){
            Bounds bounds = Colliders[0].bounds;
            for (int i = 1, ni = Colliders.Length; i < ni; i++)
            {
                bounds.Encapsulate(Colliders[i].bounds);
            }

            return bounds;
        }else{
            return new Bounds();
        }
    }
}